include(CheckCCompilerFlag)

function(CCompilerOptions_checkopts VAR)
  set(_i 1)
  while(_i LESS ${ARGC})
    list(GET ARGV ${_i} _opt)
    incr(_i)
    list(GET ARGV ${_i} _flag)

    check_c_compiler_flag(${_opt} ${_flag})
    set(_result ${${_flag}})
    if (_result)
      set(${VAR} ${_opt} PARENT_SCOPE)
      return()
    endif()
    incr(_i)
  endwhile()
endfunction(CCompilerOptions_checkopts)

macro(incr VAR)
  if (${ARGC} GREATER 1)
    math(EXPR ${VAR} "${${VAR}} + ${ARGV1}")
  else()
    math(EXPR ${VAR} "${${VAR}} + 1")
  endif()
endmacro(incr)

function(CCompilerOptions)
  set(_arglist ${ARGV})

  unset(_optlist)

  set(_i 0)
  while(_i LESS ${ARGC})
    list(GET _arglist ${_i} _arg)

    unset(_flag)
    if (${_arg} STREQUAL "LANG")
      incr(_i)
      list(GET _arglist ${_i} _arg)
      _checkopt_lang(${_arg})
    elseif(${_arg} STREQUAL "DEBUG")
      incr(_i)
      list(GET _arglist ${_i} _arg)
      _checkopt_debug(${_arg})
    elseif(${_arg} STREQUAL "WARNINGS")
      incr(_i)
      list(GET _arglist ${_i} _arg)
      _checkopt_warn(${_arg})
    endif()

    if (_flag)
      message(STATUS "... found flag ${_flag}")
      list(APPEND _optlist ${_flag})
    endif()

    incr(_i)
  endwhile()

  if(_optlist)
    foreach(_opt IN LISTS _optlist)
      set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS} ${_opt}")
    endforeach()

    set(CMAKE_C_FLAGS "${CMAKE_C_FLAGS}" PARENT_SCOPE)
  endif(_optlist)
endfunction(CCompilerOptions)

macro(_checkopt_lang_c99)
  message(STATUS "Checking for flags to specify C99...")
  CCompilerOptions_checkopts(_flag 
    "-std=c99" HAS_CLANG_STYLE_STD_C99
    "--std=c99" HAS_GCC_STYLE_STD_C99)
endmacro()

macro(_checkopt_lang_gnu99)
  message(STATUS "Checking for flags to specify C99 w/ GNU extensions...")
  CCompilerOptions_checkopts(_flag 
    "-std=gnu99" HAS_CLANG_STYLE_STD_GNU99
    "--std=gnu99" HAS_GCC_STYLE_STD_GNU99)
endmacro()

macro(_checkopt_lang_c11)
  message(STATUS "Checking for flags to specify C99...")
  CCompilerOptions_checkopts(_flag 
    "-std=c11" HAS_CLANG_STYLE_STD_C11
    "--std=c11" HAS_GCC_STYLE_STD_C11
    "-std=gnu99" HAS_CLANG_STYLE_STD_GNU99
    "--std=gnu99" HAS_GCC_STYLE_STD_GNU99
    "-std=c99" HAS_CLANG_STYLE_STD_C99
    "--std=c99" HAS_GCC_STYLE_STD_C99)
endmacro()

macro(_checkopt_lang_gnu11)
  message(STATUS "Checking for flags to specify C99...")
  CCompilerOptions_checkopts(_flag 
    "-std=gnu11" HAS_CLANG_STYLE_STD_GNU11
    "--std=gnu11" HAS_GCC_STYLE_STD_GNU11
    "-std=gnu99" HAS_CLANG_STYLE_STD_GNU11
    "--std=gnu99" HAS_GCC_STYLE_STD_GNU11
    )
endmacro()

macro(_checkopt_lang OPT)
  unset(_flag)
  if (${OPT} STREQUAL "C99")
    _checkopt_lang_c99()
  elseif (${OPT} STREQUAL "C11")
    _checkopt_lang_c11()
  else()
    message(ERROR "Unknown language option ${OPT}")
  endif()
endmacro()

macro(_checkopt_debug OPT)
  unset(_flag)
  if (${OPT} STREQUAL "VERBOSE")
    message(STATUS "Checking for flags to specify verbose debugging...")
    CCompilerOptions_checkopts(_flag 
      "-g3" HAS_GCC_STYLE_G3
      "-g"  HAS_CLANG_STYLE_G)
  elseif(${OPT} STREQUAL "ON")
    message(STATUS "Checking for flags to enable debugging...")
    CCompilerOptions_checkopts(_flag 
      "-g1" HAS_GCC_STYLE_G1
      "-g"  HAS_CLANG_STYLE_G)
  elseif(${OPT} STREQUAL "OFF")
    message(STATUS "Checking for flags to disable debugging...")
    CCompilerOptions_checkopts(_flag 
      "-g0" HAS_GCC_STYLE_G0)
  else()
    message(ERROR "Unknown debugging option ${OPT}")
  endif()
endmacro()

macro(_checkopt_warn OPT)
  unset(_flag)
  if(${OPT} STREQUAL "ALL")
    message(STATUS "Checking for flags to enable all warnings...")
    CCompilerOptions_checkopts(_flag 
      "-Wall" HAS_GCC_STYLE_W_ALL)

  elseif(${OPT} STREQUAL "EXTRA")
    message(STATUS "Checking for flags to enable extra warnings...")
    CCompilerOptions_checkopts(_flag 
      "-Wextra" HAS_GCC_STYLE_W_EXTRA)

  elseif(${OPT} STREQUAL "NONE")
    message(STATUS "Checking for flags to disable all warnings...")
    CCompilerOptions_checkopts(_flag 
      "-w" HAS_GCC_STYLE_INHIBIT_WARNINGS)
  elseif(${OPT} STREQUAL "PEDANTIC")
    message(STATUS "Checking for flags to enable pedantic warnings...")
    CCompilerOptions_checkopts(_flag 
      "-pedantic" HAS_GCC_STYLE_PEDANTIC
      "-Wpedantic" HAS_GCC_STYLE_W_PEDANTIC)
  elseif(${OPT} STREQUAL "AS_ERRORS")
    message(STATUS "Checking for flags to turn warnings into errors...")
    CCompilerOptions_checkopts(_flag 
      "-Werror" HAS_GCC_STYLE_W_ERROR)
  else()
    message(ERROR "Unknown warnings option ${OPT}")
  endif()
endmacro()


