#include "vector3.h"

void mat3x3_axis_rotate(mat3x3* mat, enum MAT3X3_ROT_AXIS axis, double ang)
{
  double cos_a = cos(ang);
  double sin_a = sin(ang);

  switch (axis)
  {
    case ROT_NONE:
      mat3x3_zero(mat);
      break;

    case ROT_X_AXIS:
      MAT3X3_ELT(mat, 0,0) = 1.0; MAT3X3_ELT(mat, 0,1) =   0.0; MAT3X3_ELT(mat, 0,2) =    0.0;
      MAT3X3_ELT(mat, 1,0) = 0.0; MAT3X3_ELT(mat, 1,1) = cos_a; MAT3X3_ELT(mat, 1,2) = -sin_a;
      MAT3X3_ELT(mat, 2,0) = 0.0; MAT3X3_ELT(mat, 2,1) = sin_a; MAT3X3_ELT(mat, 2,2) =  cos_a;
      break;

    case ROT_DX_AXIS:
      MAT3X3_ELT(mat, 0,0) = 0.0; MAT3X3_ELT(mat, 0,1) =    0.0; MAT3X3_ELT(mat, 0,2) =    0.0;
      MAT3X3_ELT(mat, 1,0) = 0.0; MAT3X3_ELT(mat, 1,1) = -sin_a; MAT3X3_ELT(mat, 1,2) = -cos_a;
      MAT3X3_ELT(mat, 2,0) = 0.0; MAT3X3_ELT(mat, 2,1) =  cos_a; MAT3X3_ELT(mat, 2,2) = -sin_a;
      break;

    case ROT_DX2_AXIS:
      MAT3X3_ELT(mat, 0,0) = 0.0; MAT3X3_ELT(mat, 0,1) =    0.0; MAT3X3_ELT(mat, 0,2) =    0.0;
      MAT3X3_ELT(mat, 1,0) = 0.0; MAT3X3_ELT(mat, 1,1) = -cos_a; MAT3X3_ELT(mat, 1,2) =  sin_a;
      MAT3X3_ELT(mat, 2,0) = 0.0; MAT3X3_ELT(mat, 2,1) = -sin_a; MAT3X3_ELT(mat, 2,2) = -cos_a;
      break;

    case ROT_Y_AXIS:
      MAT3X3_ELT(mat, 0,0) =  cos_a; MAT3X3_ELT(mat, 0,1) = 0.0; MAT3X3_ELT(mat, 0,2) = sin_a;
      MAT3X3_ELT(mat, 1,0) =    0.0; MAT3X3_ELT(mat, 1,1) = 1.0; MAT3X3_ELT(mat, 1,2) =   0.0;
      MAT3X3_ELT(mat, 2,0) = -sin_a; MAT3X3_ELT(mat, 2,1) = 0.0; MAT3X3_ELT(mat, 2,2) = cos_a;
      break;

    case ROT_DY_AXIS:
      MAT3X3_ELT(mat, 0,0) = -sin_a; MAT3X3_ELT(mat, 0,1) = 0.0; MAT3X3_ELT(mat, 0,2) =  cos_a;
      MAT3X3_ELT(mat, 1,0) =    0.0; MAT3X3_ELT(mat, 1,1) = 0.0; MAT3X3_ELT(mat, 1,2) =    0.0;
      MAT3X3_ELT(mat, 2,0) = -cos_a; MAT3X3_ELT(mat, 2,1) = 0.0; MAT3X3_ELT(mat, 2,2) = -sin_a;
      break;

    case ROT_DY2_AXIS:
      MAT3X3_ELT(mat, 0,0) = -cos_a; MAT3X3_ELT(mat, 0,1) = 0.0; MAT3X3_ELT(mat, 0,2) = -sin_a;
      MAT3X3_ELT(mat, 1,0) =    0.0; MAT3X3_ELT(mat, 1,1) = 0.0; MAT3X3_ELT(mat, 1,2) =    0.0;
      MAT3X3_ELT(mat, 2,0) =  sin_a; MAT3X3_ELT(mat, 2,1) = 0.0; MAT3X3_ELT(mat, 2,2) = -cos_a;
      break;

    case ROT_Z_AXIS:
      MAT3X3_ELT(mat, 0,0) = cos_a; MAT3X3_ELT(mat, 0,1) = -sin_a; MAT3X3_ELT(mat, 0,2) = 0.0;
      MAT3X3_ELT(mat, 1,0) = sin_a; MAT3X3_ELT(mat, 1,1) =  cos_a; MAT3X3_ELT(mat, 1,2) = 0.0;
      MAT3X3_ELT(mat, 2,0) =   0.0; MAT3X3_ELT(mat, 2,1) =    0.0; MAT3X3_ELT(mat, 2,2) = 1.0;
      break;

    case ROT_DZ_AXIS:
      MAT3X3_ELT(mat, 0,0) = -sin_a; MAT3X3_ELT(mat, 0,1) = -cos_a; MAT3X3_ELT(mat, 0,2) = 0.0;
      MAT3X3_ELT(mat, 1,0) =  cos_a; MAT3X3_ELT(mat, 1,1) = -sin_a; MAT3X3_ELT(mat, 1,2) = 0.0;
      MAT3X3_ELT(mat, 2,0) =    0.0; MAT3X3_ELT(mat, 2,1) =    0.0; MAT3X3_ELT(mat, 2,2) = 0.0;
      break;

    case ROT_DZ2_AXIS:
      MAT3X3_ELT(mat, 0,0) = -cos_a; MAT3X3_ELT(mat, 0,1) =  sin_a; MAT3X3_ELT(mat, 0,2) = 0.0;
      MAT3X3_ELT(mat, 1,0) = -sin_a; MAT3X3_ELT(mat, 1,1) = -cos_a; MAT3X3_ELT(mat, 1,2) = 0.0;
      MAT3X3_ELT(mat, 2,0) =    0.0; MAT3X3_ELT(mat, 2,1) =    0.0; MAT3X3_ELT(mat, 2,2) = 0.0;
      break;
  }
}

