#include "ts_common_algo.h"

#include <stdlib.h>
#include <string.h>

#define TOO_SMALL_FOR_QUICK_SORT 5

uint32_t djb2_hash(unsigned char *str)
{
  uint32_t hash = 5381;
  int c;

  while ( (c = *str++) != 0 )
    hash = ((hash << 5) + hash) + c; /* hash * 33 + c */

  return hash;
}

int strings_equal(const char* a, const char* b)
{
  return strcmp(a,b) == 0;
}

#define AT(ind) ((void*) ( ((char*)base) + (ind)*(wid) ))
#define SWAP(i,j) do { if (i != j) { memcpy(tmp, AT(i), wid); memcpy( AT(i), AT(j), wid ); memcpy(AT(j), tmp, wid); } } while (0)
#define CMP(i,j) ( compare(closure, AT(i), AT(j)) )
void insertion_sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare)
{
  void* tmp = ts_alloc(NO_ALLOCATOR, wid);

  for (size_t i=1; i < nelts; i++) {
    for (size_t j=0; j < i; j++) {
      if ( CMP(i,j) < 0 ) {
        memcpy(tmp, AT(i), wid);
        memmove(AT(j+1), AT(j), wid*(i-j));
        memcpy(AT(j), tmp, wid);
      }
    }
  }

  ts_free(NO_ALLOCATOR, tmp);
}

void quick_sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare)
{
  void* tmp = ts_alloc(NO_ALLOCATOR, wid);

  // choose pivot
  if (nelts <= TOO_SMALL_FOR_QUICK_SORT) {
    insertion_sort_elements(base,nelts,wid,closure,compare);
    return;
  }

  size_t pivot = nelts/2;
  size_t end = nelts-1;

  SWAP(pivot,end);

  pivot = 0;
  for (size_t i=0; i < end; i++) {
    if ( CMP(i,end) <= 0 ) {
      SWAP(i,pivot);
      pivot++;
    }
  }

  SWAP(pivot,end);

  if (pivot > 0) {
    quick_sort_elements(AT(0), pivot, wid, closure, compare);
  }
  
  if (pivot < nelts-1) {
    quick_sort_elements(AT(pivot+1), nelts-pivot-1, wid, closure, compare);
  }

  ts_free(NO_ALLOCATOR, tmp);
}
#undef AT
#undef SWAP
#undef CMP

#define AT(ind) (base + (ind))
#define SWAP(i,j) do { if (i != j) { tmp = base[(i)]; base[(i)] = base[(j)]; base[(j)] = tmp; } } while(0)
#define CMP(i,j) ( (base[(i)] > base[(j)]) - (base[(j)] > base[(i)]) )
void insertion_sort_sizes(size_t *base, size_t nelts)
{
  size_t i,j,tmp;

  for (i=1; i < nelts; i++) {
    for (j=0; j < i; j++) {
      int c = CMP(i,j);
      if ( CMP(i,j) < 0 ) {
        SWAP(i,j);
      }
    }
  }
}

void quick_sort_sizes(size_t *base, size_t nelts)
{
  size_t pivot,end,tmp;

  // choose pivot
  if (nelts <= TOO_SMALL_FOR_QUICK_SORT) {
    insertion_sort_sizes(base,nelts);
    return;
  }

  pivot = nelts/2;
  end = nelts-1;

  SWAP(pivot,end);

  pivot = 0;
  for (size_t i=0; i < end; i++) {
    if ( CMP(i,end) <= 0 ) {
      SWAP(i,pivot);
      pivot++;
    }
  }

  SWAP(pivot,end);

  if (pivot > 0) {
    quick_sort_sizes(AT(0), pivot);
  }
  
  if (pivot < nelts-1) {
    quick_sort_sizes(AT(pivot+1), nelts-pivot-1);
  }
}
#undef AT
#undef SWAP
#undef CMP

void sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare)
{
  quick_sort_elements(base,nelts,wid,closure,compare);
}

struct sort_indices_closure
{
  void* orig_closure;
  GenericCompareFunc orig_compare;

  void* elements;
  size_t wid;
};

#define AT(ind) ((void*) ( ((char*)(closure->elements)) + (ind)*(closure->wid) ))
static int compare_elements_through_indices(struct sort_indices_closure* closure, const size_t* a, const size_t* b)
{
  return closure->orig_compare(closure->orig_closure, AT(*a), AT(*b));
}
#undef AT

void sort_indices(void* base, size_t* inds, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare)
{
  struct sort_indices_closure ind_closure;

  ind_closure.orig_closure = closure;
  ind_closure.orig_compare = compare;
  ind_closure.elements = base;
  ind_closure.wid = wid;

  for (size_t i=0; i < nelts; i++) { inds[i] = i; }
  sort_elements(inds, nelts, sizeof(inds[0]), &ind_closure, (GenericCompareFunc)compare_elements_through_indices);
}

static void sizes_merge(const size_t *restrict l1, const size_t *restrict l2, size_t *restrict tmp, size_t n1, size_t n2)
{
  const size_t *restrict end1 = l1+n1;
  const size_t *restrict end2 = l2+n2;

  while ((l1<end1) && (l2<end2))
  {
    if (*l1 <= *l2) { *tmp = *l1; l1++; }
    else { *tmp = *l2; l2++; }
    tmp++;
  }

  // while (l1 < end1) { *tmp = *l1; l1++, tmp++; }
  // while (l2 < end2) { *tmp = *l2; l2++, tmp++; }
  if (l1<end1) { memcpy(tmp,l1,(end1-l1) * sizeof(*l1)); }
  else if (l2<end2) { memcpy(tmp,l2,(end2-l2) * sizeof(*l2)); }
}

size_t *merge_sort_sizes(size_t *restrict base, size_t *restrict tmp, size_t n)
{
  size_t subn = 1;
  size_t skip = 2*subn;
  size_t i;

  size_t *l1 = base;
  size_t *l2 = tmp;
  size_t *sw;
  while (subn < n)
  {
    for (i=0; i+skip <= n; i += skip)
    {
      sizes_merge(l1+i,l1+i+subn, l2+i, subn,subn);
    }
    if (i+subn<n)
    {
      size_t n2 = n - (i+subn);
      sizes_merge(l1+i,l1+i+subn, l2+i, subn,n2);
    }
    else if (i < n)
    {
      memcpy(l2+i,l1+i,(n-i)*sizeof(*l1));
    }

    sw = l1; l1 = l2; l2 = sw;

    subn = skip;
    skip *= 2;
  }

  return l1;
}

static void u32_merge(
    const uint32_t *restrict l1,
    const uint32_t *restrict l2,
    uint32_t *restrict tmp, size_t n1, size_t n2)
{
  const uint32_t *restrict end1 = l1+n1;
  const uint32_t *restrict end2 = l2+n2;

  while ((l1<end1) && (l2<end2))
  {
    if (*l1 <= *l2) { *tmp = *l1; l1++; }
    else { *tmp = *l2; l2++; }
    tmp++;
  }

  // while (l1 < end1) { *tmp = *l1; l1++, tmp++; }
  // while (l2 < end2) { *tmp = *l2; l2++, tmp++; }
  if (l1<end1) { memcpy(tmp,l1,(end1-l1) * sizeof *l1); }
  else if (l2<end2) { memcpy(tmp,l2,(end2-l2) * sizeof *l2); }
}

uint32_t *merge_sort_u32(uint32_t *restrict base, uint32_t *restrict tmp, size_t n)
{
  size_t subn = 1;
  size_t skip = 2*subn;
  size_t i;

  uint32_t *l1 = base;
  uint32_t *l2 = tmp;
  uint32_t *sw;
  while (subn < n)
  {
    for (i=0; i+skip <= n; i += skip)
    {
      u32_merge(l1+i,l1+i+subn, l2+i, subn,subn);
    }
    if (i+subn<n)
    {
      size_t n2 = n - (i+subn);
      u32_merge(l1+i,l1+i+subn, l2+i, subn,n2);
    }
    else if (i < n)
    {
      memcpy(l2+i,l1+i,(n-i)*sizeof(*l1));
    }

    sw = l1; l1 = l2; l2 = sw;

    subn = skip;
    skip *= 2;
  }

  return l1;
}

