#ifndef TS_COMMON_ALGO_H
#define TS_COMMON_ALGO_H

#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdint.h>
#include <inttypes.h>
#include <errno.h>

#include "ts_common.h"
#include "ts_errors.h"

#define MODULE TS_COMMON_ALGO

#define STRINGS_EQ(a,b) ( strcmp((a),(b)) == 0 )

#define STRLIT_EQ(s, lit) (strncmp(lit,s,sizeof(lit))==0)

uint32_t djb2_hash(unsigned char *str);

int strings_equal(const char* a, const char* b);

typedef int (*GenericCompareFunc)(void* closure, const void *, const void *);

// void sort_elements(void* base, size_t nelts, size_t eltsize, GenericCompareFunc cmp, void* closure);

void sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare);
void sort_indices(void* base, size_t* inds, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare);

void insertion_sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare);
void quick_sort_elements(void* base, size_t nelts, size_t wid, void* closure, GenericCompareFunc compare);

size_t *merge_sort_sizes(size_t *restrict base, size_t *restrict tmp, size_t n);
uint32_t *merge_sort_u32(uint32_t *restrict base, uint32_t *restrict tmp, size_t n);

void insertion_sort_sizes(size_t *base, size_t nelts);
void quick_sort_sizes(size_t *base, size_t nelts);

static inline void invert_map_array(size_t*restrict forw, size_t*restrict back, size_t nelts)
{
  for(size_t i=0; i < nelts; i++) {
    back[ forw[i] ] = i;
  }
}

static inline TSResult string_to_double(const char* s, double*restrict vp)
{
  char* endp = NULL;
  double v;

  if (s == NULL || *s == '\0') {
    return TS_NEW_ERROR(TS_ERR_NOT_A_NUMBER);
  }

  errno = 0;
  v = strtod(s,&endp);
  if (*endp != '\0') {
    return TS_NEW_ERROR(TS_ERR_NOT_A_NUMBER);
  }
  if (errno == ERANGE) {
    TS_RAISE_ERROR_MESG(TS_ERR_INVALID_VALUE, "value is too big for double");
  }
  if (errno != 0) {
    TS_RAISE_ERRORF(TS_ERR_UNKNOWN_ERROR, "unexpected error: %s", strerror(errno));
  }

  if (vp != NULL) { *vp = v; }
  return TS_OK;
}

static inline TSResult string_to_size(const char* s, size_t* szp)
{
  char *e = NULL;
  uintmax_t v;
  if (s == NULL || *s == '\0') {
    return TS_NEW_ERROR(TS_ERR_NOT_A_NUMBER);
  }

  errno = 0;
  v = strtoumax(s, &e, 0);
  if (v > SIZE_MAX || errno == ERANGE) {
    TS_RAISE_ERROR_MESG(TS_ERR_INVALID_VALUE, "value is out of range for size");
  }
  if (errno != 0) {
    TS_RAISE_ERRORF(TS_ERR_UNKNOWN_ERROR, "unexpected error: %s", strerror(errno));
  }

  *szp = (size_t)v;
  return TS_OK;
}

#undef MODULE

#endif /* TS_COMMON_ALGO_H */

