/* return codes above this offset are errors */
#ifndef TS_ERROR_OFFSET
#  define TS_ERROR_OFFSET 0x0010
#endif 

_EC( TS_OKAY                                    , 0x0000, "no errors"                                   )

// Status updates -- not actual errors
_EC( TS_STOP_ITERATION                          , 0x0001, "stop iteration"                              )
_EC( TS_CANCELLED                               , 0x0002, "operation cancelled"                         )

_EC( TS_IO_EOF                                  , 0x0003, "end of file at valid state"                  )

// **** ERRORS ****

// General errors
_EC( TS_ERR_NULL_POINTER_ARG                    , 0x0010, "argument passed was a NULL pointer"          )
_EC( TS_ERR_SIMULATION_STATE                    , 0x0011, "error in simulation state"                   )
_EC( TS_ERR_UNSUPPORTED_OPERATION               , 0x0012, "unsupported operation"                       )
_EC( TS_ERR_NOT_A_NUMBER                        , 0x0013, "not a number"                                )
_EC( TS_ERR_INVALID_ARGUMENT                    , 0x0014, "invalid argument"                            )
_EC( TS_ERR_INVALID_RETURN                      , 0x0015, "invalid return"                              )
_EC( TS_ERR_INVALID_INDEX                       , 0x0016, "invalid index"                               )
_EC( TS_ERR_INVALID_VALUE                       , 0x0017, "invalid value"                               )
_EC( TS_ERR_MAXIMUM_ALLOCATED                   , 0x0018, "maximum number of objects allocated"         )
_EC( TS_ERR_UNIMPLEMENTED                       , 0x0019, "unimplemented"                               )
_EC( TS_ERR_INSUFFICIENT_SIZE                   , 0x0020, "insufficient size"                           )
_EC( TS_ERR_IN_ALGORITHM                        , 0x0021, "error encountered in algorithm"              )
_EC( TS_ERR_INVALID_PARAM_NAME                  , 0x0022, "invalid parameter name"                      )

_EC( TS_ERR_IN_RUBY                             , 0x0101, "error in Ruby script"                        )
_EC( TS_ERR_IN_PYTHON                           , 0x0102, "error in Python script"                      )

_EC( TS_ERR_NO_SUCH_PARAMETER                   , 0x1001, "no such parameter"                           )
_EC( TS_ERR_ADDING_PARAMETER                    , 0x1002, "error adding parameter"                      )

_EC( TS_ERR_NO_SUCH_BONDTYPE                    , 0x1011, "no such bond type"                           )
_EC( TS_ERR_ADDING_BONDTYPE                     , 0x1012, "error adding bond type"                      )

_EC( TS_ERR_UNSUPPORTED_BOND_PATTERN            , 0x1013, "unsupported bond pattern"                    )

_EC( TS_ERR_SHAKE_BOND_TOO_LONG                 , 0x2001, "SHAKE: bond is too long"                     )
_EC( TS_ERR_SHAKE_MAX_ITERATIONS                , 0x2002, "SHAKE: maximum iterations exceeded"          )

// Potential errors (0xB0Dx)
_EC( TS_ERR_INCORRECT_NUM_BODIES                , 0xB0D1, "Incorrect number of bodies for potential"    )
_EC( TS_ERR_UNSUPPORTED_POTENTIAL               , 0xB0D2, "Unsupported potential"                       )
_EC( TS_ERR_UNKNOWN_POTENTIAL_PARAM             , 0xB0D3, "Unknown potential parameter"                 )

// Connectivity errors (0xBBxy)
_EC( TS_ERR_PARTICLES_ALREADY_BONDED            , 0xBB01, "Particles already have a bond"               )
_EC( TS_ERR_PARTICLES_NOT_BONDED                , 0xBB02, "Particles are not bonded"                    )

// Configuration errors
_EC( TS_ERR_UNKNOWN_INTEGRATOR                  , 0xBC01, "Integrator is unknown"                       )
_EC( TS_ERR_BAD_DT                              , 0xBC02, "Invalid timestep"                            )
_EC( TS_ERR_BAD_TEMP                            , 0xBC03, "Invalid temperature"                         )

// Out of memory error
_EC( TS_ERR_OUT_OF_MEMORY                       , 0xD1E0, "System is out of memory"                     )

// Error initializing the thread pool
_EC( TS_ERR_THREAD_POOL                         , 0xD1F0, "Error initializing the threaded pool"        )

// Chemical model errors
_EC( TS_ERR_LATT_NO_BARRIER_MODEL               , 0xC0C1, "Lattice: No barrier selected"                )
_EC( TS_ERR_LATT_UNIMPLEMENTED_BARRIER_MODEL    , 0xC0C2, "Lattice: Barrier model not implemented"      )
_EC( TS_ERR_LATT_BAD_STATE                      , 0xC0C3, "Lattice: bad state encountered"              )
_EC( TS_ERR_LATT_RATE_IS_ZERO                   , 0xC0C4, "Lattice: Total rate is zero"                 )
_EC( TS_ERR_LATT_PSEUDOSPIN_NOT_REGISTERED      , 0xC0C5, "Lattice: pseudospin not registered"          )
_EC( TS_ERR_LATT_DUPLICATE_MOVE                 , 0xC0C6, "Lattice: duplicate state in moves for site"  )

_EC( TS_ERR_CHEMDRIVER_INCONSISTENT_ROW_RANGE   , 0xC0D1, "ChemDriver: row range cache is inconsistent" )
_EC( TS_ERR_CHEMDRIVER_BARRIER_MODEL_CONFLICT   , 0xC0D2, "ChemDriver: barrier model conflicts with another setting ")

// Various subsystem errors
_EC( TS_ERR_DISPATCH_INITIALIZATION             , 0xDE00, "Error in initializing dispatch subsystem"    )
_EC( TS_ERR_DISPATCH_WORKER                     , 0xDE01, "Error in dispatch worker"                    )
_EC( TS_ERR_DISPATCH_DISABLED                   , 0xDE02, "Dispatch is disabled"                        )

_EC( TS_ERR_MINIMIZATION_ERROR                  , 0xEF01, "Error encountered in minimizer"              )
_EC( TS_ERR_MINIMIZATION_INCOMPLETE             , 0xEF02, "Minimizer did not converge"                  )

// RNG is not assigned
_EC( TS_ERR_NO_RNG                              , 0xF0E1, "Simulation requires a random number generator")
_EC( TS_ERR_WRONG_RNG                           , 0xF0E2, "Simulation has the wrong RNG"                )
_EC( TS_ERR_RNG_INCONSISTENT_STATE              , 0xF0E3, "RNG is in an inconsistent state"             )

// Force-loop errors
_EC( TS_ERR_IN_FORCE_LOOP                       , 0xF001, "Error encountered in the force loop"         )

// I/O errors
_EC( TS_ERR_IO_ERROR                            , 0xFE01, "I/O: error occurred"                         )
_EC( TS_ERR_READ_NOT_FINISHED                   , 0xFE02, "I/O: read not finished"                      )
_EC( TS_ERR_WRITE_NOT_FINISHED                  , 0xFE03, "I/O: write not finished"                     )
_EC( TS_ERR_IO_DECODE_ERROR                     , 0xFE04, "I/O: error decoding stream"                  )
_EC( TS_ERR_IO_ENCODE_ERROR                     , 0xFE05, "I/O: error encoding stream"                  )

_EC( TS_ERR_INSUFFICIENT_ARRAY_SIZE             , 0xFEA4, "Insufficient array size"                     )
_EC( TS_ERR_INVALID_ARRAY_SIZE                  , 0xFEA5, "Invalid array size"                          )
_EC( TS_ERR_INVALID_MATRIX_SIZE                 , 0xFEA6, "Invalid matrix size"                         )

_EC( TS_ERR_PARSE_ERROR                         , 0xFEC0, "error in parsing"                            )
_EC( TS_ERR_CONFIG_MISSING_SECTION              , 0xFEC1, "a required configuration value is missing"   )
_EC( TS_ERR_CONFIG_MISSING_VALUE                , 0xFEC2, "a required configuration value is missing"   )
_EC( TS_ERR_CONFIG_INVALID_VALUE                , 0xFEC3, "a required configuration value is invalid"   )
_EC( TS_ERR_CONFIG_DUPLICATE_VALUE              , 0xFEC4, "duplicate configuration entries"             )
_EC( TS_ERR_WRONG_NUMBER_OF_ARGUMENTS           , 0xFEC5, "wrong number of arguments given to a function" )

_EC( TS_ERR_YAML_LOOKAHEAD_EXCEEDED             , 0XFE11, "YAML: lookahead exceeded"                    )
_EC( TS_ERR_YAML_INVALID_CONFIG_DOC             , 0xFE12, "YAML: invalid configuration file"            )
_EC( TS_ERR_YAML_INVALID_KEY                    , 0xFE13, "YAML: invalid key"                           )
_EC( TS_ERR_YAML_INVALID_VALUE                  , 0xFE14, "YAML: invalid key"                           )
_EC( TS_ERR_YAML_INVALID_SECTION                , 0xFE15, "YAML: invalid section"                       )

_EC( TS_ERR_YAML_INVALID_PTYPE                  , 0xFE21, "YAML: invalid particle type"                 )
_EC( TS_ERR_YAML_INVALID_PTYPE_LIST             , 0xFE22, "YAML: invalid particle type list"            )

_EC( TS_ERR_YAML_INVALID_RBTYPE                 , 0xFE23, "YAML: invalid rigid body type"               )
_EC( TS_ERR_YAML_INVALID_RBTYPE_LIST            , 0xFE24, "YAML: invalid rigid body type list"          )

_EC( TS_ERR_YAML_INVALID_BONDTYPE               , 0xFE31, "YAML: invalid bond type"                     )
_EC( TS_ERR_YAML_INVALID_BONDTYPE_LIST          , 0xFE32, "YAML: invalid bond type list"                )

_EC( TS_ERR_YAML_INVALID_DYNAMICS_SECTION       , 0xFE41, "YAML: invalid dynamics section"              )
_EC( TS_ERR_YAML_INVALID_DYNAMICS_KEY           , 0xFE42, "YAML: invalid key in dynamics section"       )
_EC( TS_ERR_YAML_INVALID_DYNAMICS_VALUE         , 0xFE43, "YAML: invalid value in dynamics section"     )

_EC( TS_ERR_YAML_INVALID_PARAMETERS_SECTION     , 0xFE51, "YAML: invalid parameter section"             )

_EC( TS_ERR_YAML_INVALID_VECTOR3                , 0xFE61, "YAML: invalid 3-D vector"                    )

_EC( TS_ERR_YAML_INVALID_BONDED_PATTERN         , 0xFE71, "YAML: invalid bonded pattern"                )

_EC( TS_ERR_BAD_FILE_FORMAT                     , 0xFEF1, "Bad file format"                             )
_EC( TS_ERR_UNKNOWN_FILE_VERSION                , 0xFEF2, "Unknown file version"                        )
_EC( TS_ERR_FILE_IS_BYTESWAPPED                 , 0xFEF3, "File is byteswapped"                         )
_EC( TS_ERR_INVALID_FILE_SECTION                , 0xFEF4, "Invalid file section"                        )

_EC( TS_ERR_LOGIC_ERROR                         , 0xFFF1, "Logic error encountered"                     )

_EC( TS_ERR_UNKNOWN_ERROR                       , 0xFFFF, "Unknown error"                               )

