#ifndef TS_FENWICK_H
#define TS_FENWICK_H

#include <stdlib.h>
#include "ts_errors.h"

/*
struct ts_fenwick_element
{
  double count;
  void* data;
};
*/

struct ts_fenwick_index_lookup; /* opaque type for index_lookup table */

typedef struct ts_fenwick_tree ts_fenwick_tree;
struct ts_fenwick_tree
{
  size_t capacity;
  size_t size;

  size_t size_2n;  /* Smallest power of two that is >= max_elements */

  double total_sum;

  double *counts;
  void **data;
  // struct ts_fenwick_element* elements;

  struct ts_fenwick_index_lookup* index_lookup;
};

TSResult ts_fenwick_tree_init(ts_fenwick_tree* tree, size_t max);
TSResult ts_fenwick_tree_finalize(ts_fenwick_tree* tree);
TSResult ts_fenwick_tree_clear(ts_fenwick_tree* tree, void (*cleanup_func)(void* closure, void* data), void* closure);

TSResult ts_fenwick_tree_resize(ts_fenwick_tree* tree, size_t new_max);

TSResult ts_fenwick_tree_add(ts_fenwick_tree* tree, double freq, void* data);
TSResult ts_fenwick_tree_remove_at(ts_fenwick_tree* tree, size_t ind);
TSResult ts_fenwick_tree_replace_at(ts_fenwick_tree* tree, size_t ind, double freq, void* data);
TSResult ts_fenwick_tree_update_count(ts_fenwick_tree* tree, size_t ind, double count);
TSResult ts_fenwick_tree_get_frequency(ts_fenwick_tree* tree, size_t ind, double* freqp);

TSResult ts_fenwick_tree_get_cum_sum(ts_fenwick_tree* tree, 
    size_t ind, double* cum_freq);

size_t ts_fenwick_tree_find_cum_sum(ts_fenwick_tree* tree, double sum);
size_t ts_fenwick_tree_find(ts_fenwick_tree* tree, void* data);

#endif /* TS_FENWICK_H */

