#ifndef TS_INTEGRATORS__H
#define TS_INTEGRATORS__H

#include "tubesheet.h"

#define MODULE_NAME TS_INTEGRATORS

static inline int ts_integrator_is_brownian(const ts_sim* sim)
{
  return (sim->integrator == TS_INTEGRATOR_BROWNIAN_DYNAMICS)
    ||   (sim->integrator == TS_INTEGRATOR_STOCH_RUNGE_KUTTA);
}

TSResult ts_sim_single_step(ts_sim* sim);
TSResult ts_sim_many_steps(ts_sim* sim, size_t nsteps);
TSResult ts_sim_step_until(ts_sim* sim, double t_end);

typedef struct ts_integrator_callback ts_integrator_callback;
typedef TSResult (*TSIntegratorCallbackFunc)(ts_sim* sim, ts_integrator_callback* closure);
struct ts_integrator_callback
{
  TSIntegratorCallbackFunc callback;

  int freq;
  int countdown;
};

TSResult ts_add_callback(ts_sim* sim, ts_integrator_callback* callback);
TSResult ts_run_callbacks(ts_sim* sim);


#undef MODULE_NAME

#endif /* TS_INTEGRATORS__H */

