#include "ts_io.h"
#include "tsif.pb.h"
#include "pb_encode.h"
#include "pb_decode.h"
#include "ts_dof_map.h"

struct tsif_arg
{
  ts_sim* sim;
  struct tsif_state* state;
  struct ts_dof_map dof_map;
};

static const char tmp_val[] = "TMP";
static bool tmp_encode(pb_ostream_t* stream)
{
  return pb_write(stream, (const uint8_t*)tmp_val, sizeof(tmp_val));
}

static bool tmp_decode(pb_istream_t* stream)
{
  char buf[sizeof(tmp_val)];

  return pb_read(stream, (uint8_t*)buf, sizeof(buf));
}

MAYBE_UNUSED static bool
bonds_encode(pb_ostream_t* stream, const pb_field_t* field, void* const *arg)
{
  UNUSED_PARAM(field);
  UNUSED_PARAM(arg);
  // struct tsif_data* data = *arg;

  if (!pb_encode_tag(stream, PB_WT_STRING, field->tag))
    return false;

  return tmp_encode(stream);
  /* encode free particle bonds */
  /*
{
  for (size_t fi=0; fi < dof_map->num_free_particles; fi++)
  {
    size_t i = dof_map->fp_to_p[fi];
    ts_particle* p = ts_get_particle(sim,i);

    for (size_t b=0; b < p->num_bonds; b++)
    {
      // Ensures that particle index of p is the particle that p is
      // bonded to.  Uses the bond info invariant: pi < pj.
      if (p == p->bonds[b].pi)
      {
        RETURN_ON_ERROR3( write_bond_pair(sim,f, dof_map, p->bonds+b) );
      }
    }
  }
  }
  */

  /* encode rigid body bonds */
  /*
{
  for (size_t i=0; i < sim->num_rigid_bodies; i++)
  {
    ts_rigid_body* ri = ts_get_rigid_body(sim,i);
    for (size_t j=i+1; j < sim->num_rigid_bodies; j++)
    {
      ts_rigid_body* rj = ts_get_rigid_body(sim,j);

      struct ts_rigid_body_bond_info binfo;
      if (ts_rigid_body_find_first_bond(sim, ri,rj, &binfo))
      {
        RETURN_ON_ERROR3( write_rigid_pair(sim,f, ri,rj, &binfo) );
      }
    }
  }

}
  */

  return true;
}

MAYBE_UNUSED static bool
bonds_decode(pb_istream_t* stream, const pb_field_t* field, void** arg)
{
  UNUSED_PARAM(field);
  UNUSED_PARAM(arg);
  return tmp_decode(stream);
}

MAYBE_UNUSED static bool
tmp_encode_callback(pb_ostream_t* stream, const pb_field_t* field, void* const *arg)
{
  UNUSED_PARAM(arg);
  if (!pb_encode_tag(stream, PB_WT_STRING, field->tag))
    return false;

  return tmp_encode(stream);
}

MAYBE_UNUSED static bool
tmp_decode_callback(pb_istream_t* stream, const pb_field_t* field, void** arg)
{
  UNUSED_PARAM(field);
  UNUSED_PARAM(arg);
  return tmp_decode(stream);
}

/*
static bool ppos_decode(pb_istream_t* stream, const pb_field_t* field, void** arg)
{
}

static bool rpos_decode(pb_istream_t* stream, const pb_field_t* field, void** arg)
{
}
*/

static bool file_writer(pb_ostream_t* os, const uint8_t* buf, size_t count)
{
  FILE* f = (FILE*) os->state;
  return fwrite(buf,sizeof(*buf),count,f) == count;
}

static bool file_reader(pb_istream_t* is, uint8_t* buf, size_t count)
{
  FILE* f = (FILE*) is->state;
  bool status;

  if (buf == NULL)
  {
    while (count) {
      if (fgetc(f) == EOF) break;
      count--;
    }
    return count == 0;
  }

  status = (fread(buf, sizeof(*buf), count, f) == count);
  if (feof(f)) {
    is->bytes_left = 0;
  }

  return status;
}

static pb_ostream_t pb_ostream_from_file(FILE* f)
{
  pb_ostream_t os = {
    .callback = &file_writer,
    .state = f,
    .max_size = SIZE_MAX,
    0
  };

  return os;
}

static pb_istream_t pb_istream_from_file(FILE* f)
{
  pb_istream_t is = {
    .callback = &file_reader,
    .state = f,
    0
  };

  return is;
}

TSResult tsif_read_state(FILE* f, struct tsif_state* state, ts_allocator* A)
{
  UNUSED_PARAM(state);
  UNUSED_PARAM(A);

  // struct tsif_arg arg = { .sim = NULL, .state = state, .dof_map = { 0 } }; 

  ts_pb_state pb_state;
  memset(&pb_state,0,sizeof(pb_state));

  /*
  pb_state.bonds.funcs.decode = bonds_decode;
  pb_state.bonds.arg = &arg;

  pb_state.particle_pos.coord.funcs.decode = tmp_decode_callback;
  pb_state.particle_pos.coord.arg = state;

  pb_state.rigid_pos.coord.funcs.decode = tmp_decode_callback;
  pb_state.rigid_pos.coord.arg = state;
  */

  pb_istream_t is = pb_istream_from_file(f);

  if (!pb_decode(&is, ts_pb_state_fields, &pb_state))
    return TS_NEW_ERROR2(TS_ERR_IO_DECODE_ERROR,
        ("Error decoding TSIF: %s", PB_GET_ERROR(&is)));

  return TS_OK;
}

TSResult tsif_write_state(ts_sim* sim, FILE* f)
{
  struct tsif_arg arg = { .sim = sim, .state = NULL }; 

  RETURN_ON_ERROR( ts_dof_map_init(&arg.dof_map, sim) );

  ts_pb_state pb_state;
  memset(&pb_state,0,sizeof(pb_state));

  pb_state.sim_state.nparticles = sim->num_particles;
  pb_state.sim_state.time = sim->time;
  pb_state.sim_state.temp = sim->temp;
  pb_state.sim_state.PE   = sim->PE;
  pb_state.sim_state.has_KE = 0;
  /* pb_state.sim_state.KE = sim->KE; */
  pb_state.sim_state.totE = sim->totalE;

  /*
  pb_state.bonds.funcs.encode = bonds_encode;
  pb_state.bonds.arg = &arg;

  pb_state.particle_pos.coord.funcs.encode = tmp_encode_callback;
  pb_state.particle_pos.coord.arg = &arg;

  pb_state.rigid_pos.coord.funcs.encode = tmp_encode_callback;
  pb_state.rigid_pos.coord.arg = &arg;
  */

  pb_ostream_t os = pb_ostream_from_file(f);

  bool succ = pb_encode(&os, ts_pb_state_fields, &pb_state);

  ts_dof_map_finalize(&arg.dof_map);

  if (succ)
    return TS_OK;
  else
    return TS_NEW_ERROR2(TS_ERR_IO_ENCODE_ERROR,
        ("Error encoding TSIF: %s", PB_GET_ERROR(&os)));
}

