#ifndef TS_JSON_H
#define TS_JSON_H

#include <stdio.h>
#include <stdlib.h>

#include "ts_errors.h"

enum TS_JSON_EVENT {
  TS_JSON_NONE = 0,

  TS_JSON_LITERAL ,
  TS_JSON_STRING  ,
  TS_JSON_NUMBER  ,

  TS_JSON_ARRAY_BEGIN,
  TS_JSON_ARRAY_END,

  TS_JSON_OBJECT_BEGIN,
  TS_JSON_OBJECT_KEY,
  TS_JSON_OBJECT_END,

  TS_JSON_EOF,

  TS_JSON_EVENT_MAX
};

enum TS_JSON_LITERAL {
  TS_JSON_NULL  = -1,
  TS_JSON_FALSE =  0,
  TS_JSON_TRUE  =  1,
};

struct ts_json_string { char* p; size_t len; };
union ts_json_value
{
  double num;
  struct ts_json_string str;
  enum TS_JSON_LITERAL lit;
  int val;
};

typedef struct ts_json_event ts_json_event;
struct ts_json_event
{
  enum TS_JSON_EVENT type;
  union ts_json_value value;
};

enum TS_JSON_INPUT_TYPE {
  TS_JSON_INPUT_NONE = 0,
  TS_JSON_INPUT_BUF  = 1,
  TS_JSON_INPUT_CSTR = 2,
  TS_JSON_INPUT_FILE = 3,
};

enum TS_JSON_PARSER_STATE {
  TS_JSON_PARSER_BARE = 0,
  TS_JSON_PARSER_STRING  ,
  TS_JSON_PARSER_LITERAL ,
  TS_JSON_PARSER_NUMBER  ,

  TS_JSON_PARSER_ARRAY_OBJ,
  TS_JSON_PARSER_ARRAY_DELIM,

  TS_JSON_PARSER_OBJECT_KEY,
  TS_JSON_PARSER_OBJECT_KV_DELIM,
  TS_JSON_PARSER_OBJECT_VALUE,
  TS_JSON_PARSER_OBJECT_PAIR_DELIM,
};

#define TS_JSON_STACKSIZE 128
typedef struct ts_json_parser ts_json_parser;
struct ts_json_parser
{
  FILE* f;

  char* buf;
  size_t blen;
  size_t bpos;
  size_t spos;

  int (*parser_getc)(ts_json_parser*);
  void (*parser_ungetc)(ts_json_parser*,int);

  int enable_literal_strings;

  int prev_line;
  int prev_col;

  int line;
  int col;

  int stack_top;
  enum TS_JSON_PARSER_STATE state[TS_JSON_STACKSIZE];

  void* free_func_data;
  void (*free_func)(void*,void*);
};

TSResult ts_json_init_buf(ts_json_parser* parser,
    char* p, size_t len,
    void (*freefunc)(void*,void*),void* cl);
void ts_json_finalize(ts_json_parser* parser);

TSResult ts_json_next(ts_json_parser* parser, ts_json_event* event);

#endif /* TS_JSON_H */
