#include "ts_machdeps.h"

#include <string.h>
#include "ts_malloc_protect.h"

#if defined(__APPLE__)
int is_file_readonly(FILE* f)
{
  return (f->_flags & __SRD) > 0;
}

int is_file_writeonly(FILE* f)
{
  return (f->_flags & __SWR) > 0;
}

int is_file_readwrite(FILE* f)
{
  return (f->_flags & __SRW) > 0;
}

int is_file_readable(FILE* f)
{
  return is_file_readonly(f) || is_file_readwrite(f);
}

int is_file_writeable(FILE* f)
{
  return is_file_writeonly(f) || is_file_readwrite(f);
}
#endif /* defined(__APPLE__) */

#if !defined(_WIN32) && (defined(__unix__) || defined(__unix) || (defined(__APPLE__) && defined(__MACH__)))
#  if !defined(_POSIX_SOURCE)
#    define _POSIX_SOURCE
#  endif /* !defined(_POSIX_SOURCE) */

#include <signal.h>

static struct sigaction prev_handler;

void enable_user_interrupt(void)
{
  struct sigaction sig;
  sig.sa_handler = SIG_DFL;
  sigemptyset(&sig.sa_mask);
  sig.sa_flags = 0;

  memset(&prev_handler,0,sizeof(prev_handler));

  /* FIXME: needs proper error handling */
  sigaction(SIGINT, &sig, &prev_handler);
}

void restore_user_interrupt(void)
{
  /* FIXME: needs proper error handling */
  sigaction(SIGINT, &prev_handler, NULL);
}

#else /* !defined(_WIN32) && etc. */

void enable_user_interrupt(void) {} 
void restore_user_interrupt(void) {}

#endif /* !defined(_WIN32) && etc. */
