#ifndef TS_MACHDEPS__H
#define TS_MACHDEPS__H

#include <stdio.h>
#include <stdint.h>

#include "vector3.h"
#if TS_USE_AVX
#  include <immintrin.h>

/* _mm256_set_m128d is missing in clang's headers */
#define ts_mm256_set_m128d(/* __m128d */ hi, /* __m128d */ lo) \
    _mm256_insertf128_pd(_mm256_castpd128_pd256(lo), (hi), 0x1)

#endif

#if TS_USE_SSE
#  include <xmmintrin.h>
#  include <emmintrin.h>
#  include <pmmintrin.h>
#endif /* TS_USE_SSE */


#define MODULE_NAME TS_MACHDEPS

#if __STDC_VERSION__ >= 201112L
#  define HAS_ALIGNOF
#  include <stdalign.h>
enum { DBL_ALIGN = alignof(double) };
enum { FLT_ALIGN = alignof(float) };

enum { U64_ALIGN = alignof(uint64_t) };
enum { S64_ALIGN = alignof(int64_t) };

enum { U32_ALIGN = alignof(uint32_t) };
enum { S32_ALIGN = alignof(int32_t) };

enum { U16_ALIGN = alignof(uint16_t) };
enum { S16_ALIGN = alignof(int16_t) };
#else
/* this is a bit of a hack to test for the alignment
 * of doubles
 */
struct dbl_align { char x; double d; };
struct flt_align { char x; float v;    };

struct u64_align { char x; uint64_t v; };
struct s64_align { char x; int64_t v;  };

struct u32_align { char x; uint32_t v; };
struct s32_align { char x; int32_t v;  };

struct u16_align { char x; uint16_t v; };
struct s16_align { char x; int16_t v;  };

enum { DBL_ALIGN = offsetof(struct dbl_align,d) };
enum { FLT_ALIGN = offsetof(struct flt_align,v) };

enum { U64_ALIGN = offsetof(struct u64_align,v) };
enum { S64_ALIGN = offsetof(struct s64_align,v) };

enum { U32_ALIGN = offsetof(struct u32_align,v) };
enum { S32_ALIGN = offsetof(struct s32_align,v) };

enum { U16_ALIGN = offsetof(struct u16_align,v) };
enum { S16_ALIGN = offsetof(struct s16_align,v) };
#if 0
#endif
#endif

#if defined(__i386__) || defined(__x86_64__) && defined(__GNUCC__)
#  define TS_BREAKPOINT() do { __asm__ __volatile__ ("int $03"); } while(0)
#  define PREFETCH(x) (__builtin_prefetch(x))
#else
#  define TS_BREAKPOINT() ERROR__TS_BREAKPOINT__NOT_SUPPORTED__ON_SYSTEM
#  define PREFETCH(x)
#endif


int is_file_readonly(FILE *f);
int is_file_writeonly(FILE *f);
int is_file_readwrite(FILE *f);

int is_file_readable(FILE *f);
int is_file_writeable(FILE *f);

/* Very simplified interface to enabling the default interrupt handler
 * and restoring the previous handler.  
 *
 * On POSIX systems:
 *   enable_user_interrupt
 *     sets the SIGINT signal handler to be default, saving the previous
 *     handler.
 *
 *   restore_user_interrupt
 *     restores the previous handler.
 */
void enable_user_interrupt(void);
void restore_user_interrupt(void);

#undef MODULE_NAME

#endif /* TS_MACHDEPS__H */

