#ifndef TS_MINIMIZE_H
#define TS_MINIMIZE_H

#include "tubesheet.h"

#define MODULE_NAME TS_MINIMIZE

enum TS_MINIMIZE_METHOD {
  TS_MINIMIZE_DEFAULT     = 0,
  TS_MINIMIZE_STEEPEST    = 1,
  TS_MINIMIZE_LBFGS       = 2,

  TS_MINIMIZE_MAXMETHOD
};

enum TS_MINIMIZE_WHAT {
  TS_MINIMIZE_POS = 1,
  TS_MINIMIZE_ORI = 2,

  TS_MINIMIZE_ALL = TS_MINIMIZE_POS | TS_MINIMIZE_ORI,
};

struct ts_minimize_params {
  enum TS_MINIMIZE_METHOD method;
  enum TS_MINIMIZE_WHAT   what;

  int rescale_dofs;
  int use_nonbonds;

  int max_iter;
  int report_freq;

  int dim_subsp;

  double mineig;

  VecReal eps; 

  VecReal minstep;
  VecReal maxstep;

  VecReal prec;

  VecReal step0;
};

void ts_minimize_initialize_params(struct ts_minimize_params* params);

TSResult ts_minimize_energy(ts_sim* sim, const struct ts_minimize_params* params);
TSResult ts_minimize_lbfgs(ts_sim* sim, const struct ts_minimize_params* params);
TSResult ts_minimize_steepest(ts_sim* sim, const struct ts_minimize_params* params);

typedef struct ts_minimization_protocol ts_minimization_protocol;
enum { TS_MINIMIZATION_PROTOCOL_DEFAULT_SIZE = 2 };
struct ts_minimization_protocol
{
  ts_minimization_protocol* next;
  int nminimizations;
  struct ts_minimize_params params[TS_MINIMIZATION_PROTOCOL_DEFAULT_SIZE];
};

TSResult ts_minimization_protocol_run(ts_sim* sim, const ts_minimization_protocol* protocol);

#undef MODULE_NAME

#endif /* TS_MINIMIZE_H */

