#ifndef TS_RNG__H
#define TS_RNG__H

#include <stdlib.h>
#include <stdint.h>
#include "dSFMT/dSFMT.h"

#include "ts_errors.h"

#define MODULE_NAME TS_RNG

typedef TSResult (*RNG_call_func)(size_t n, double* vals, void* state);
typedef TSResult (*RNG_print_func)(void* state, FILE* f);
typedef TSResult (*RNG_free_func)(void* state);

enum TS_RNG_TYPE {
  TS_RNG_UNDEFINED = 0,
  TS_RNG_TEST      = 1,
  TS_RNG_MT19937   = 2,
};

#define TS_RNG_NEXTRA 4
struct ts_rng {
  enum TS_RNG_TYPE type;

  RNG_free_func rng_free;
  RNG_print_func rng_print;

  RNG_call_func gen_uniforms;
  void* state;

  RNG_call_func gen_normals;

  int gauss_has_extra;
  double gauss_extra;
};

TSResult ts_rng_gen_uniforms(struct ts_rng* state, size_t n, double* v);
TSResult ts_rng_gen_normals(struct ts_rng* state, size_t n, double* v);
TSResult ts_rng_free(struct ts_rng* state);

TSResult ts_rng_print_configuration(const struct ts_rng* state, FILE* f);

TSResult RNG_mt19937_new(uint32_t seed, struct ts_rng** rngp);
TSResult RNG_mt19937_get_info(struct ts_rng* state, uint32_t* seed);

TSResult RNG_test_new(size_t n, double* vals, struct ts_rng** rngp);
TSResult RNG_test_get_info(struct ts_rng* state, size_t* seq_len, size_t* seq_idx, double** seq_vals);

TSResult ts_rng_gen_normals_sequential(struct ts_rng* state, size_t n, double* v);
TSResult ts_rng_gen_normals_block(struct ts_rng* state, size_t n, double* v);

#undef MODULE_NAME

#endif /* TS_RNG__H */

