#ifndef TS_TABLES_PRIV_H
#define TS_TABLES_PRIV_H

#include "khash.h"

#define MODULE_NAME TS_TABLES_PRIV

#define TABLE_WRAPPERS(tag,prefix,htype,vtype)                  \
  static inline MAYBE_UNUSED htype * SYMCAT(prefix,_table_new)(void) {       \
    return (htype *)kh_init(tag);                               \
  }                                                             \
                                                                \
  static inline MAYBE_UNUSED void SYMCAT(prefix,_table_delete)(htype * tbl) {\
    kh_destroy(tag, (khash_t(tag)*)tbl);                        \
  }                                                             \
  static inline MAYBE_UNUSED void SYMCAT(prefix,_table_clear)(htype * tbl) { \
    kh_clear(tag, (khash_t(tag)*)tbl);                          \
  }                                                             \
  static inline MAYBE_UNUSED int SYMCAT(prefix,_table_exists)(htype * tbl, const char* key) { \
    return kh_get(tag, (khash_t(tag)*)tbl, key) != kh_end( (khash_t(tag)*)tbl ); \
  }                                                             \
  static inline MAYBE_UNUSED int SYMCAT(prefix,_table_get)(htype * tbl,      \
      const char* key, vtype * valp) {                          \
    khiter_t iter = kh_get(tag, (khash_t(tag)*)tbl, key);       \
    if (iter != kh_end((khash_t(tag)*)tbl)) {                   \
      *valp = kh_value((khash_t(tag)*)tbl, iter); return 1;     \
    } else { return 0; }                                        \
  }                                                             \
  static inline MAYBE_UNUSED int SYMCAT(prefix,_table_replace)(htype * tbl,  \
      const char *key, vtype * valp) {                          \
    vtype prev; khiter_t iter;                                  \
    iter = kh_get(tag, (khash_t(tag)*)tbl, key);                \
    if (iter == kh_end((khash_t(tag)*)tbl)) { return 0; }       \
    prev = kh_value((khash_t(tag)*)tbl,iter);                   \
    kh_value((khash_t(tag)*)tbl,iter) = *valp;                  \
    *valp = prev;                                               \
    return 1;                                                   \
  }                                                             \
  static inline MAYBE_UNUSED int SYMCAT(prefix,_table_set)(htype * tbl,      \
      const char* key, vtype val) {                             \
    int ret = 0;                                                \
    khiter_t iter = kh_put(tag, (khash_t(tag)*)tbl, key, &ret); \
    if (ret < 0) return 0;                                      \
    kh_value( (khash_t(tag)*)tbl, iter ) = val;                 \
    return 1;                                                   \
  }                                                             \
  static inline MAYBE_UNUSED size_t SYMCAT(prefix,_table_size(htype * tbl)) {\
    return kh_size( (khash_t(tag)*) tbl );                      \
  }                                                             \
  static inline MAYBE_UNUSED void SYMCAT(prefix,_table_remove)(htype * tbl, const char* key) { \
    khint_t ind = kh_get(tag,(khash_t(tag)*)tbl, key);          \
    if (ind != kh_end( (khash_t(tag)*)tbl )) {                  \
      kh_del(tag, (khash_t(tag)*)tbl, ind);                     \
    }                                                           \
  }                                                             \
  DEFINITION_EAT_SEMI

#define DEFINE_TABLE(_tag, _prefix, _htype, _vtype)        \
  KHASH_MAP_INIT_STR(_tag, _vtype)                         \
  struct _htype { khash_t(_tag) SYMCAT(_prefix,_table); }; \
  TABLE_WRAPPERS(_tag, _prefix, struct _htype, _vtype)

#undef MODULE_NAME

#endif /* TS_TABLES_PRIV_H */

