#ifndef TS_YAML_H
#define TS_YAML_H

#include "tubesheet.h"
#include "ts_common_algo.h"
#include "ts_bonds.h"

#include <yaml.h>

#define MODULE_NAME TS_YAML

typedef void (*autorelease_pool_finalize_func)(void* closure, void* data);
struct autorelease_pool {
  struct autorelease_pool* prev;
  data_vector elements;
  autorelease_pool_finalize_func finalizer;
  void* finalizer_closure;
};

struct autorelease_pool* tsy_autorelease_pool_new(struct autorelease_pool* prev, 
    size_t elt_size, size_t n,
    autorelease_pool_finalize_func finalizer, 
    void* finalizer_closure);

#define TS_NEW_YAML_ERROR(err, evt, mesg, ...)     \
  TS_NEW_ERROR2(err,                            \
      ("YAML error at line %d, column %d: " mesg, \
       (evt)->start_mark.line,                  \
       (evt)->start_mark.column,                \
       __VA_ARGS__))

struct ts_yaml_parser
{
  yaml_parser_t* parser;
  yaml_event_t* lookahead_event;

  struct autorelease_pool* event_pool;
  struct autorelease_pool* string_pool;

  int print_event_indent;
};

void ts_yaml_parser_init(struct ts_yaml_parser* yp, yaml_parser_t* parser);
void ts_yaml_parse_finalize(struct ts_yaml_parser* yp);
const char* ts_yaml_event_type_name(yaml_event_type_t type);

static inline struct autorelease_pool*
tsy_get_event_pool(struct ts_yaml_parser* yp) { return yp->event_pool; }

static inline struct autorelease_pool*
tsy_get_string_pool(struct ts_yaml_parser* yp) { return yp->string_pool; }

static inline const char*
ts_yaml_event_name(const yaml_event_t* evt)
{
  return ts_yaml_event_type_name(evt->type);
}

TSResult tsy_get_event(struct ts_yaml_parser* yp, yaml_event_t** event_ptr);
TSResult tsy_unget_event(struct ts_yaml_parser* yp, yaml_event_t* event);

#define TSY_GET_EVENT(yp,e) RETURN_ON_ERROR( tsy_get_event((yp),&(e)) )
#define TSY_UNGET_EVENT(yp,e) RETURN_ON_ERROR( tsy_unget_event((yp),(e)) )

static inline TSResult
tsy_expect_event(struct ts_yaml_parser* yp, yaml_event_type_t etype, enum TS_ERROR_CODE ecode)
{
  yaml_event_t* expected_event = NULL;
  TSY_GET_EVENT(yp, expected_event);

  if (expected_event->type == etype) { return TS_OK; }

  return TS_NEW_YAML_ERROR(ecode, expected_event,
      "expected event type %s but found %s",
      ts_yaml_event_type_name(etype),
      ts_yaml_event_name(expected_event));
}

#define TSY_EXPECT_EVENT(p,etype,err) \
  RETURN_ON_ERROR( tsy_expect_event((p),(etype),(err)) )

static inline TSResult tsy_get_scalar(struct ts_yaml_parser* yp, yaml_event_t** event_ptr, enum TS_ERROR_CODE notScalarECode)
{
  TSY_GET_EVENT(yp,*event_ptr);
  if ((*event_ptr)->type == YAML_SCALAR_EVENT) { return TS_OK; }
  return TS_NEW_ERROR(notScalarECode);
}

#define TSY_GET_SCALAR(p,e,err) \
  RETURN_ON_ERROR(tsy_get_scalar((p),&(e),(err)))

void tsy_print_event(struct ts_yaml_parser* yp, yaml_event_t* evt);

#define TSY_SCALAR_STRING(evt) ((char*)((evt)->data.scalar.value))
#define TSY_SCALAR_EQ(evt,str) ( strcmp(TSY_SCALAR_STRING(evt), (str)) == 0 )

#undef MODULE_NAME

#endif /* TS_YAML_H */

