class IOError < StandardError; end
class EOFError < IOError; end

class IO
  def self.open(*args, &block)
    io = self.new(*args)

    return io unless block

    begin
      yield io
    ensure
      begin
        io.close unless io.closed?
      rescue StandardError
      end
    end
  end

  def print(*args)
    args.each { |elt| write elt.to_s }
  end

  def putc(o)
    case o
    when Integer
      raise RangeError, "integer must be in [0,255]" if (o < 0) || (o > 255)
      write o.chr

    when String
      raise ArgumentError, "string must not be empty" unless o[0]
      write o[0]
    else
      raise TypeError, "expected integer or string"
    end

    self
  end

  def print(*args)
    count = 0
    args.each do |elt|
      elt = "nil" if elt.nil?
      count += write(elt)
    end
    return count
  end

  def puts(*args)
    return write "\n" if args.empty?

    count = 0
    args.each do |elt|
      if Array === elt
        elt.each do |itm|
          count += puts( (elt == itm) ? "[...]" : itm.to_s )
        end
      else
        if elt.nil?
          count += write "nil"
        else
          elt = elt.to_s
          unless String === elt
            raise RuntimeError,
              "could not coerce object #{elt} to String with to_s"
          end
          count += write(elt)
          count += write "\n" unless elt[-1] == "\n"
        end
      end
    end
    return count
  end

end

module Kernel
  def print(*args)
    STDOUT.print(*args)
  end

  def puts(*args)
    STDOUT.puts(*args)
  end

  def printf(*args)
    STDOUT.printf(*args)
  end

  def gets(*args)
    STDIN.gets(*args)
  end

  def getc(*args)
    STDIN.getc(*args)
  end
end

