#include "tsmrb_classes.h"

#include "mruby/variable.h"

static const char* classes_tbl_name = "tsmrb classes";

struct tsmrb*
tsmrb_classes(mrb_state* mrb)
{
  mrb_sym tbl = mrb_intern_cstr(mrb, classes_tbl_name);
  mrb_value val = mrb_gv_get(mrb, tbl);

  if (MRB_TT_CPTR != mrb_type(val))
    mrb_raise(mrb, E_ARGUMENT_ERROR, "invalid classes table value");

  return mrb_cptr(val);
}

void tsmrb_init(mrb_state* mrb)
{
  mrb_sym tblsym = mrb_intern_cstr(mrb, classes_tbl_name);
  if (!mrb_nil_p(mrb_gv_get(mrb, tblsym)))
    return;

  struct tsmrb* ctbl = mrb_malloc(mrb, sizeof(*ctbl));
  memset(ctbl, 0, sizeof(*ctbl));

  mrb_value ctbl_val = mrb_cptr_value(mrb, ctbl);
  mrb_gv_set(mrb, tblsym, ctbl_val);
}

static inline void def_sym_ent(mrb_state* mrb, struct tsmrb_sym_ent* ent, const char* s, size_t slen)
{
  ent->name = s;
  ent->sym  = mrb_intern_static(mrb,s,slen);
}

void tsmrb_symtab_init(mrb_state* mrb)
{
  struct tsmrb_sym_ent* symtab = tsmrb_symtab(mrb);
#define X(n) def_sym_ent(mrb, symtab+TSMRB_SYM_IND(n), #n, sizeof(#n)-1);
#define XS(sym,str) def_sym_ent(mrb, symtab+sym, str, sizeof(str)-1);
#  include "tsmrb_symtab_entries.h"
#undef X
#undef XS
}




