#ifndef TSMRB_CLASSES_H
#define TSMRB_CLASSES_H

#include "ts_common.h"
#include "mruby.h"
#include "mruby/class.h"

struct tsmrb_sym_ent
{
  const char* name;
  mrb_sym sym;
};

struct tsmrb_rate_prefactor {
  mrb_sym sym;
  int evtype;
};

#define X(n) SYMCAT(TSMRBSYM_,n),
#define XS(sym,str) sym,
enum {
  TSMRB_SYM_NULL_ENTRY = 0,
#include "tsmrb_symtab_entries.h"
  TSMRB_SYM_MAXENTRY,
};
#undef X
#undef XS

struct tsmrb
{
  ts_allocator* mrb_alloc;

  struct RClass* eSimError;
  struct RClass* cMemory;

  struct RClass* mTubesheet;
  struct RClass* mNumerical;
  struct RClass* mPotentials;
  struct RClass* mConvenience;
  struct RClass* mUnits;

  struct RClass* cVec3;
  struct RClass* cMat3x3;
  struct RClass* cQuat;

  struct RClass* cSparseBuilder;
  struct RClass* cSparseMatrix;

  struct RClass* cPoly;

  struct RClass* cVectorList;

  struct RClass* cJSON;

  struct RClass* cRNG;

  struct RClass* cSim;
  struct RClass* cParticle;
  struct RClass* cRigidBody;

  struct RClass* cCallback;

  struct RClass* cPotential;
  struct RClass* cTwoBody;
  struct RClass* cThreeBody;
  struct RClass* cFourBody;

  struct RClass* cConstantField;
  struct RClass* cSpinAroundParticle;
  struct RClass* cPlaneBarrier;

  struct RClass* cHarmonic;
  struct RClass* cWCA;

  struct RClass* cAngleHarmonic;

  struct RClass* cDihedral;
  struct RClass* cDipole;

  struct RClass* cFenwickTree;
  struct RClass* cLattice;
  struct RClass* cChemDriver;
  struct RClass* cSheetLengthBias;
  struct RClass* cFastSheetModel;
  struct RClass* cChemConfigure;

  struct RClass* mIO;
  struct RClass* cTSMF;

  mrb_value tblNamedPotentials;

  struct bond_list_term* bond_list_terms;
  struct tsmrb_rate_prefactor *rate_prefactor_table;
  struct tsmrb_sym_ent symtab[TSMRB_SYM_MAXENTRY];
};

void tsmrb_init(mrb_state* mrb);
struct tsmrb* tsmrb_classes(mrb_state* mrb);

static inline struct tsmrb_sym_ent*
tsmrb_symtab(mrb_state* mrb)
{
  return tsmrb_classes(mrb)->symtab;
}

#define TSMRB_SYM_IND(n) SYMCAT(TSMRBSYM_,n)
#define TSMRB_SYM(n) (tsmrb_symtab(mrb)[TSMRB_SYM_IND(n)].sym)
#define TSMRB_SYM_OBJ(n) (mrb_symbol_value(tsmrb_symtab(mrb)[TSMRB_SYM_IND(n)].sym))

static inline mrb_sym
tsmrb_symtab_at(mrb_state* mrb, int ind)
{
  if ((ind < 0) || (ind >= TSMRB_SYM_MAXENTRY))
    mrb_raisef(mrb, E_RANGE_ERROR,
        "invalid symtab index %S", mrb_fixnum_value(ind));

  return tsmrb_symtab(mrb)[ind].sym;
}

void tsmrb_symtab_init(mrb_state* mrb);

#endif /* TSMRB_CLASSES_H */
