#include "tsmrb_stats.h"

#include <stdlib.h>
#include <string.h>

#include "mruby/array.h"

#include "tsmrb_helpers.h"

enum {
  MEAN_ELT = 0,
  VAR_ELT  ,
  COUNT_ELT,
  MIN_ELT  ,
  MAX_ELT  ,
  STAT_ARY_LEN
};

static inline mrb_float ary_to_float(mrb_state* mrb, mrb_value ary, int i)
{
  mrb_value elt = mrb_ary_ref(mrb, ary, i);
  if (!mrb_numeric_p(elt))
    tsmrb_raisef(mrb, E_TYPE_ERROR, "element %d expected to be numeric", i);
  return mrb_to_float(mrb, elt);
}

static inline mrb_int ary_to_int(mrb_state* mrb, mrb_value ary, int i)
{
  mrb_value elt = mrb_ary_ref(mrb, ary, i);
  if (!mrb_numeric_p(elt))
    tsmrb_raisef(mrb, E_TYPE_ERROR, "element %d expected to be numeric", i);
  return mrb_to_fixnum(mrb, elt);
}

void tsmrb_accumulated_stats_from_ary(mrb_state* mrb, struct ts_accumulated_stats* stats, mrb_value r_stats)
{
  if (RARRAY_LEN(r_stats) == 0)
    memset(stats,0,sizeof(*stats));

  if (RARRAY_LEN(r_stats) != STAT_ARY_LEN)
    tsmrb_raisef(mrb,E_ARGUMENT_ERROR, "initialized stat array must have %d elements", STAT_ARY_LEN);

  stats->mean  = ary_to_float(mrb, r_stats, MEAN_ELT);
  stats->var   = ary_to_float(mrb, r_stats, VAR_ELT );
  stats->count = ary_to_int(  mrb, r_stats, COUNT_ELT);
  stats->min   = ary_to_float(mrb, r_stats, MIN_ELT );
  stats->max   = ary_to_float(mrb, r_stats, MAX_ELT );
}

mrb_value tsmrb_accumulated_stats_to_ary(mrb_state* mrb, struct ts_accumulated_stats* stats)
{
  mrb_value elts[STAT_ARY_LEN];

  for(int i=0; i < STAT_ARY_LEN; i++)
    elts[i] = mrb_nil_value();

  elts[MEAN_ELT ] = mrb_float_value(mrb, stats->mean);
  elts[VAR_ELT  ] = mrb_float_value(mrb, stats->var );
  elts[COUNT_ELT] = mrb_fixnum_value( stats->count );
  elts[MIN_ELT  ] = mrb_float_value(mrb, stats->min);
  elts[MAX_ELT  ] = mrb_float_value(mrb, stats->max);

  return mrb_ary_new_from_values(mrb, STAT_ARY_LEN, elts);
}

void tsmrb_accumulated_stats_set_ary(mrb_state* mrb, struct ts_accumulated_stats* stats, mrb_value r_stats)
{
  // Reverse order... only resize an empty array once
  mrb_ary_set(mrb, r_stats, MAX_ELT  , mrb_float_value(mrb,stats->max));
  mrb_ary_set(mrb, r_stats, MIN_ELT  , mrb_float_value(mrb,stats->min));
  mrb_ary_set(mrb, r_stats, COUNT_ELT, mrb_fixnum_value(stats->count));
  mrb_ary_set(mrb, r_stats, VAR_ELT  , mrb_float_value(mrb,stats->var));
  mrb_ary_set(mrb, r_stats, MEAN_ELT , mrb_float_value(mrb,stats->mean));
}

static mrb_value mTubesheet_accumulate_stats(mrb_state* mrb, mrb_value mod)
{
  mrb_value r_stats = mrb_nil_value();
  mrb_float new_val = 0.0;
  mrb_get_args(mrb, "Af", &r_stats, &new_val);

  struct ts_accumulated_stats stats;
  if (RARRAY_LEN(r_stats) == 0)
  {
    ts_accumulated_stats_init(&stats, new_val);
  }
  else
  {
    tsmrb_accumulated_stats_from_ary(mrb, &stats, r_stats);
    ts_accumulate_stats(&stats, new_val);
  }

  return tsmrb_accumulated_stats_to_ary(mrb, &stats);
}

static mrb_value mTubesheet_accumulate_statsB(mrb_state* mrb, mrb_value module)
{
  mrb_value r_stats = mrb_nil_value();
  mrb_float new_val = 0.0;
  mrb_get_args(mrb, "Af", &r_stats, &new_val);

  struct ts_accumulated_stats stats;
  if (RARRAY_LEN(r_stats) == 0)
  {
    ts_accumulated_stats_init(&stats, new_val);
  }
  else
  {
    tsmrb_accumulated_stats_from_ary(mrb, &stats, r_stats);
    ts_accumulate_stats(&stats, new_val);
  }

  tsmrb_accumulated_stats_set_ary(mrb, &stats, r_stats);
  return r_stats;
}

struct RClass* tsmrb_define_stat_accumulator(mrb_state* mrb, struct RClass* module)
{
  mrb_define_module_function(mrb, module, "accumulate_stats", mTubesheet_accumulate_stats, MRB_ARGS_REQ(2));
  mrb_define_module_function(mrb, module, "accumulate_stats!", mTubesheet_accumulate_statsB, MRB_ARGS_REQ(2));

  return module;
}

