#include "tsmrb_units.h"

#include "tsmrb_classes.h"

#include "ts_units.h"
#include "ts_macros.h"

static inline mrb_value scale_num_by(mrb_state* mrb, mrb_value mod, double scl)
{
  mrb_float f = 0.0;
  mrb_get_args(mrb, "f", &f);
  return mrb_float_value(mrb, f * scl);
}

#define CONVERSION( name, factor ) \
  static mrb_value SYMCAT(mUnits_,name) (mrb_state* mrb, mrb_value mod) \
  { return scale_num_by(mrb,mod, (factor)); } \
  DEFINITION_EAT_SEMI 

CONVERSION( length_to_nm    , UNIT_LENGTH_NANOMETERS  );
CONVERSION( length_to_meters, UNIT_LENGTH_METERS      );
CONVERSION( mass_to_kD      , UNIT_MASS_KILODALTONS   );
CONVERSION( temp_to_K       , UNIT_TEMPERATURE_KELVIN );

CONVERSION( energy_to_kJ_per_mol, UNIT_ENERGY_KJMOL );
CONVERSION( energy_to_J_per_mol , UNIT_ENERGY_JMOL  );

CONVERSION( time_to_ns  , UNIT_TIME_NANOSECS );
CONVERSION( time_to_secs, UNIT_TIME_SECS     );

CONVERSION( vel_to_nm_per_ns, UNIT_VEL_NM_PER_NS );
CONVERSION( diff_to_sq_nm_per_ns, UNIT_DIFFUSION_SQNM_PER_NS );

struct RClass* tsmrb_define_units(mrb_state* mrb, struct RClass* mod)
{
  struct RClass* mUnits = mrb_define_module_under(mrb, mod, "Units");
  tsmrb_classes(mrb)->mUnits = mUnits;

  mrb_define_module_function(mrb, mUnits, "length_to_nm", mUnits_length_to_nm, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "length_to_meters", mUnits_length_to_meters, MRB_ARGS_REQ(1));

  mrb_define_module_function(mrb, mUnits, "mass_to_kD", mUnits_mass_to_kD, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "temp_to_K", mUnits_temp_to_K, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "energy_to_kJ_per_mol", mUnits_energy_to_kJ_per_mol, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "energy_to_J_per_mol", mUnits_energy_to_J_per_mol, MRB_ARGS_REQ(1));

  mrb_define_module_function(mrb, mUnits, "time_to_ns", mUnits_time_to_ns, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "time_to_secs", mUnits_time_to_secs, MRB_ARGS_REQ(1));

  mrb_define_module_function(mrb, mUnits, "vel_to_nm_per_ns", mUnits_vel_to_nm_per_ns, MRB_ARGS_REQ(1));
  mrb_define_module_function(mrb, mUnits, "diff_to_sq_nm_per_ns", mUnits_diff_to_sq_nm_per_ns, MRB_ARGS_REQ(1));

  mrb_define_const(mrb, mUnits, "L0_NM", mrb_float_value(mrb, UNIT_LENGTH_NANOMETERS));
  mrb_define_const(mrb, mUnits, "TAU0_NS", mrb_float_value(mrb, UNIT_TIME_NANOSECS));
  mrb_define_const(mrb, mUnits, "EPS0_KJMOL", mrb_float_value(mrb, UNIT_ENERGY_KJMOL));
  mrb_define_const(mrb, mUnits, "TEMP0_K", mrb_float_value(mrb, UNIT_TEMPERATURE_KELVIN));
  mrb_define_const(mrb, mUnits, "M0_KD", mrb_float_value(mrb, UNIT_MASS_KILODALTONS));
  mrb_define_const(mrb, mUnits, "HARMONIC_FREE_ENERGY_COEFF", mrb_float_value(mrb, HARMONIC_FREE_ENERGY_COEFF));

  return mUnits;
}

