module ChemAnalysis
  MER_ST = Tubesheet::ChemDriver::MONOMER_GTP_STRAIGHT
  MER_SD = Tubesheet::ChemDriver::MONOMER_GDP_STRAIGHT

  MER_L  = Tubesheet::ChemDriver::MONOMER_GTP_BENT_LEFT
  MER_R  = Tubesheet::ChemDriver::MONOMER_GTP_BENT_RIGHT

  MER_CT = Tubesheet::ChemDriver::MONOMER_GTP_CURVED
  MER_CD = Tubesheet::ChemDriver::MONOMER_GDP_CURVED

  IFC_S  = Tubesheet::ChemDriver::INTERFACE_STRAIGHT
  IFC_B  = Tubesheet::ChemDriver::INTERFACE_BENT
  IFC_C  = Tubesheet::ChemDriver::INTERFACE_CURVED

  IFC_TU = Tubesheet::ChemDriver::INTERFACE_TUBELIKE
  IFC_SH = Tubesheet::ChemDriver::INTERFACE_SHEETLIKE

  def die mesg
    puts <<-eos
    #{mesg}
Aborting.
    eos
    Kernel.exit 1
  end

  def is_bent?(spin)
    return spin == MER_L || spin == MER_R || spin == IFC_B
  end

  def is_straight?(spin)
    return spin == MER_ST || spin == MER_SD
  end

  def is_curved?(spin)
    return spin == MER_CT || spin == MER_CD
  end

  def is_rotated?(driver,i,j)
    (li,lj) = driver.wrap_coords(i,j-1)
    (ri,rj) = driver.wrap_coords(i,j+1)
    return (driver[li,lj] == IFC_TU && driver[ri,rj] != IFC_SH) || \
      (driver[li,lj] != IFC_SH && driver[ri,rj] == IFC_TU)
  end

  def lsqfit(xv,yv)
    xavg = xv.reduce(&:+).to_f / xv.size
    yavg = yv.reduce(&:+).to_f / yv.size
    xv1 = xv.map{|x| x-xavg}
    yv1 = yv.map{|y| y-yavg}
    nxvar = xv1.map{|x| x*x}.reduce(&:+)
    ncov = xv1.zip(yv1).map{|x,y| x*y}.reduce(&:+)

    slope = ncov / nxvar
    intercept = yavg - xavg*slope
    [slope,intercept]
  end

  def calc_mean(a)
    a.reduce { |x,y| x + y }.to_f / a.size
  end

  def calc_mean_and_sd(data)
    mean = calc_mean(data)

    delta = data.map{ |x| x - mean }
    var = delta.map{ |x| x**2 }.reduce { |x,y| x + y } -
      (delta.reduce{ |x,y| x + y })**2
    var = var / (data.size-1)

    [ mean, Math::sqrt(var) ]
  end

  def aggregate(agg, datum)
    datum.each do |k,v|
      agg[k] = [] unless agg[k]
      agg[k] << v
    end
  end
end

