require 'tubesheet'
require 'tubulin'
require 'coupling'

module Tubesheet::Configure
  class ConfigLatticeDSL
    def initialize(latt)
      @lattice = latt
    end

    def rng(args)
      if args.is_a? ::Tubesheet::RNG
        @lattice.rng = args
      elsif args.is_a? Hash
        @lattice.rng = Tubesheet::RNG.new args
      else
        raise RuntimeError.new 'rng requires either an RNG object or a hash of parameters'
      end
    end

    def temp(newTemp)
      @lattice.temp = newTemp
    end
  end

  class LatticePainter
    attr_reader :latt, :r, :c

    def initialize(latt)
      @latt = latt
      @r = 0
      @c = 0
    end

    def first_row(r)
      @r = r
    end

    def first_col(c)
      @c = c
    end

    def origin(r,c)
      @r = r
      @c = c
    end

    def convert(s)
      converted = []
      s.each_char do |ch|
        if (ch == '.') || (ch == ' ')
          converted << 0
        elsif ('0'..'9').include?(ch)
          converted << ch.to_i
        else
          converted << ch
        end
      end

      converted
    end

    def row(*sl)
      s = sl.join('')
      max_col = @latt.ncol - @c
      if s.length > max_col
        raise ArgumentError, "cannot paint more than %d columns" % [ max_col ]
      end

      convert(s).each_with_index do |ch,i|
        @latt[@r,@c+i] = ch
      end

      @r += 1
    end

    def col(*sl)
      s = sl.join('')
      max_row = @latt.nrow - @r
      if s.length > max_row
        raise ArgumentError, "cannot paint more than %d rows" % [ max_row ]
      end

      convert(s).each_with_index do |ch,i|
        @latt[@r+i,@c] = ch
      end

      @c += 1
    end

  end

end

module Tubesheet
  class Lattice
    def configure(&script)
      Tubesheet::Configure::ConfigLatticeDSL.new(self).instance_eval(&script)
    end

    def paint(&script)
      Tubesheet::Configure::LatticePainter.new(self).instance_eval(&script)
    end

  end
end


