#ifndef NUMARRAY_MRUBY_H
#define NUMARRAY_MRUBY_H

#include <stddef.h>
#include <mruby.h>

#define MODULE_NAME NUMARRAY_MRUBY

enum { MAX_DIMS = 10 };

enum MATRIX_ERROR_TYPE
{
  MATRIX_ERROR_NONE                  = 0,
  MATRIX_ERROR_DIMS_INCOMPATIBLE        ,
  MATRIX_ERROR_DIMS_CANNOT_SELF_MULTIPLY,
};

enum MATRIX_CMP_OP
{
  MATRIX_CMP_EQ = 1,
  MATRIX_CMP_NE,
  MATRIX_CMP_LE,
  MATRIX_CMP_LT,
  MATRIX_CMP_GE,
  MATRIX_CMP_GT,
  MATRIX_CMP_SPACESHIP,
};

enum SLICE_TYPE
{
  SLICE_NONE = 0,
  SLICE_ALL  ,
  SLICE_INDEX,
  SLICE_RANGE,
  SLICE_LIST ,

  SLICE_MAX_TYPE
};

struct numarray_slice
{
  enum SLICE_TYPE type;
  int nelts;

  union {
    struct {
      int beg;
      int end;
      int incr;
    } range;

    int* list;
    int index;
  } slice;
};

typedef struct numarray numarray;
struct numarray
{
  size_t nelts;
  double *elts;
  int ndims;
  int dims[MAX_DIMS];
};

mrb_value numarray_new_filled(mrb_state *mrb,
    int ndims, int *dims, double val, numarray **arr_ptr);

mrb_value numarray_new_uninitialized(mrb_state *mrb,
    int ndims, int *dims, numarray **arr_ptr);

numarray* numarray_unwrap(mrb_state* mrb, mrb_value obj);

#undef MODULE_NAME

#endif /* NUMARRAY_MRUBY_H */

