#ifndef RMPI_H
#define RMPI_H

#include <mpi.h>
#include <assert.h>

#define MODULE_NAME RMPI

enum {
  RMPI_BUFLIST_DEFAULT_SIZE = 64
};

enum {
  RMPI_BUF_NO_DATA = 0,
  RMPI_BUF_PENDING_REQUEST,
  RMPI_BUF_FREED,
};

struct rmpi_request;

struct rmpi_buffer
{
  unsigned char *ptr;
  size_t size, capa, refcount;

  union {
    // struct rmpi_request *req;
    struct rmpi_buffer *next;  /* for freelist */
  } aux;
  unsigned int lock_count;
  unsigned int aux_type;
};

enum {
  RMPI_BUFLIST_MINSIZE = 16,
};

struct rmpi_buflist
{
  struct rmpi_buflist *next;
  struct rmpi_buffer *freelist;
  size_t n,capa;
  struct rmpi_buffer bufs[];
};

struct rmpi_buffer_obj
{
  struct rmpi_buffer *buf;
};

void rmpi_buflist_compact(void);
size_t rmpi_buflist_max(void);
size_t rmpi_buflist_count(void);

int rmpi_buflist_allocate(size_t n);

struct rmpi_buffer *rmpi_buflist_new_buffer(size_t capa);

void rmpi_buffer_finalize(struct rmpi_buffer *buf);

struct rmpi_buffer *rmpi_buffer_lookup(size_t i);

static inline void rmpi_buffer_incref(struct rmpi_buffer *buf)
{
  buf->refcount++;
}

static inline void rmpi_buffer_decref(struct rmpi_buffer *buf)
{
  assert( buf->refcount > 0 );
  if (--buf->refcount == 0) {
    rmpi_buffer_finalize(buf);
  }
}

static inline int rmpi_buffer_is_locked(struct rmpi_buffer *buf)
{
  return buf->lock_count > 0;
}

static inline int rmpi_buffer_lock(struct rmpi_buffer *buf)
{
  /* FIXME: need to check for overflow, and do it in a safe, portable way */
  buf->lock_count++;

  return 1; /* should return if lock overflowed or not */
}

static inline void rmpi_buffer_unlock(struct rmpi_buffer *buf)
{
  if (buf->lock_count > 0) { buf->lock_count--; }
}

typedef struct rmpi_request rmpi_request_t;
struct rmpi_request {
  struct rmpi_request *next;
  MPI_Request req;
  struct rmpi_buffer *buf;
};

enum { RMPI_REQUEST_BLOCK_SIZE = 64 };
struct rmpi_request_block {
  struct rmpi_request_block *next;
  size_t n;
  rmpi_request_t block[RMPI_REQUEST_BLOCK_SIZE];
};

rmpi_request_t *rmpi_request_new(struct rmpi_buffer *buf);
void rmpi_request_free(void *p);
void rmpi_request_finish_waiting(void);

int rmpi_request_is_finished(rmpi_request_t *req, int wait);

#undef MODULE_NAME

#endif /* RMPI_H */

